/*
 *  BrowserControl.h
 *  BaseFunction
 *
 *  Created by pyang on 05-8-4.
 *  Copyright 2005 __Arcsoft__. All rights reserved.
 *
 */
#ifndef _H_TREECONTROL_
#define _H_TREECONTROL_

#pragma once

typedef void*			TreeNodeRef;
typedef const void*		CFTreeNodeRef;
typedef const void *	(*TreeNodeRetainCallBack)(const void *pData);
typedef void			(*TreeNodeReleaseCallBack)(const void *pData);
struct TreeNodeData
{
	void* pData;
	TreeNodeRetainCallBack	pRetainProc;
	TreeNodeReleaseCallBack	pReleaseProc;
};


EXTERN_API(TreeNodeRef)			TreeNodeCreate(const TreeNodeData* pTreeNodeData);// can be NULL

// TreeNodeRef will retain the IconRef
EXTERN_API(void)				SetTreeNodeIcon(TreeNodeRef pTreeNodeRef,IconRef pIconRef);
EXTERN_API(IconRef)				GetTreeNodeIcon(TreeNodeRef pTreeNodeRef);

// TreeNodeRef will retain the CFStringRef
EXTERN_API(void)				SetTreeNodeText(TreeNodeRef pTreeNodeRef,CFStringRef pCFStringRef);
EXTERN_API(CFStringRef)			GetTreeNodeText(TreeNodeRef pTreeNodeRef);

// TreeNodeRef will use retain the TreeNodeData->pRetainProc retain pData;
EXTERN_API(void)				SetTreeNodeData(TreeNodeRef pTreeNodeRef,const TreeNodeData* pTreeNodeData);
EXTERN_API(void)				GetTreeNodeData(TreeNodeRef pTreeNodeRef,TreeNodeData* pTreeNodeData);

EXTERN_API(CFTreeNodeRef)		RetainTreeNode(CFTreeNodeRef pTreeNodeRef);
EXTERN_API(void)				ReleaseTreeNode(CFTreeNodeRef pTreeNodeRef);

typedef	void*		TreeControlRef;
enum
{
	kHeader		   		= 1 << 1,	// has header button
	kDragAndDrop   		= 1 << 2,	// support drag and drop
	kContextMenu	   	= 1 << 3,	// has context menu
	kFullHilite			= 1 << 4,	// HiliteStyle
	kNoVerScroller		= 1 << 5,	// no ver scroller
	kNoHorScroller		= 1 << 6,	// no horz scroller
};
typedef UInt32	TreeControlStyle;

typedef OSType                          TreeControlPropertyType;
enum {
	kTreeControlIconType          = 'icnr', /* IconRef*/
	kTreeControlTextType          = 'text', /* CFStringRef */
	kTreeControlIconAndTextType   = 'ticn' /* IconRef, CFStringRef, etc */
};


typedef UInt32 TreeControlPropertyID;
enum {
	/* Predefined attribute properties, optional & non-display unless otherwise stated */
	kTreeControlIsActiveProperty = 1L, /* Boolean typed data (defaults to true) */
	kTreeControlItemIsSelectableProperty = 2L, /* Boolean typed data (defaults to true) */
	kTreeControlItemIsEditableProperty = 3L, /* Boolean typed data (defaults to false, used for editable properties) */
	kTreeControlItemIsContainerProperty = 4L, /* Boolean typed data (defaults to false) */
	kTreeControlContainerIsOpenableProperty = 5L, /* Boolean typed data (defaults to true) */
	kTreeControlContainerIsClosableProperty = 6L /* Boolean typed data (defaults to true) */
};
typedef UInt32 TreeControlNodeNotification;
enum {
	kTreeControlEditStarted       = 3,    /* Starting an EditText session for specified item */
	kTreeControlEditStopped       = 4,    /* Stopping an EditText session for specified item */
	kTreeControlItemSelected      = 5,    /* Item has just been added to the selection set */
	kTreeControlItemDoubleClicked = 7,
	kTreeControlContainerOpened   = 8,    /* Container is open */
//	kTreeControlContainerClosing  = 9,    /* Container is about to close (and will real soon now, y'all) */
	kTreeControlContainerClosed   = 10,   /* Container is closed (y'all come back now!) */
};

// Normal TreeControl Callback
// if callback return false, we should call default process 
typedef Boolean (*TreeControlNodeGetPropertyProc)(TreeControlRef pTreeControlRef,TreeControlPropertyID propertyID,TreeNodeRef pTreeNodeRef,Boolean* pEnabled);
typedef Boolean	(*TreeControlNodeNotifyProc)(TreeControlRef pTreeControlRef,TreeControlNodeNotification notifyMessage,TreeNodeRef pTreeNodeRef,void* pParam);
typedef Boolean	(*TreeControlItemCompareProc)(TreeControlRef pTreeControlRef,TreeNodeRef pTreeNodeOne,TreeNodeRef pTreeNodeTwo);
// if you don't care of contextMenu, you don't need set this
typedef Boolean	(*TreeControlGetContextMenuProc)(TreeControlRef pTreeControlRef,TreeNodeRef pTreeNodeRef,MenuRef* pMenuRef);
typedef Boolean	(*TreeControlSelectContextMenuProc)(TreeControlRef pTreeControlRef,MenuRef pContextualMenu,MenuItemIndex menuItem);
// if you dont care of Drag and Drop, you don't need set this
typedef Boolean	(*TreeControlAcceptDragProc)(TreeControlRef pTreeControlRef,DragReference theDrag,TreeNodeRef pTreeNodeRef,Boolean* pEnabled);
typedef Boolean (*TreeControlReceiveDragProc)(TreeControlRef pTreeControlRef,DragReference theDrag,TreeNodeRef pTreeNodeRef,Boolean* pEnabled);
typedef Boolean (*TreeControlAddDragItemProc)(TreeControlRef pTreeControlRef,TreeNodeRef pTreeNodeRef,ItemReference* itemRef,Boolean* pEnabled);


struct TreeControlCallbacks 
{
	TreeControlNodeGetPropertyProc		pTreeControlGetNodePropertyProc;
//	TreeControlNodeSetPropertyProc		pTreeControlSetNodePropertyProc;
	TreeControlNodeNotifyProc			pTreeControlNodeNotifyProc;
	TreeControlItemCompareProc			pTreeControlItemCompareProc;
	TreeControlGetContextMenuProc		pTreeControlGetContextMenuProc;
	TreeControlSelectContextMenuProc	pTreeControlSelectContextMenuProc;
	TreeControlAcceptDragProc			pTreeControlAcceptDragProc;
	TreeControlReceiveDragProc			pTreeControlReceiveDragProc;
	TreeControlAddDragItemProc			pTreeControlAddDragItemProc;
};

EXTERN_API(TreeControlRef)	CreateCustomTreeControl(WindowRef pWindowRef,Rect* pRect,TreeControlStyle style,TreeControlPropertyType property);
EXTERN_API(TreeControlRef)	CreateListControl(WindowRef pWindowRef,Rect* pRect,TreeControlStyle style,TreeControlPropertyType property);

//Scrolls an node into view, optionally bringing a particular part of that item into view.
EXTERN_API(OSStatus)		RevealTreeControlNode(TreeControlRef pTreeControlRef,TreeNodeRef pTreeNodeRef,Boolean bSelectNode);
EXTERN_API(OSStatus)		SetTreeControlScrollerPosition(TreeControlRef pTreeControlRef,UInt32 ver,UInt32 hor);
EXTERN_API(OSStatus)		GetTreeControlScrollerPosition(TreeControlRef,UInt32* ver,UInt32* hor);
// if you pass CFTreeNodeRef pParentNode NULL, you should get root item count
EXTERN_API(UInt32)			GetTreeControlNodeCount(TreeControlRef pTreeControlRef,CFTreeNodeRef pParentNode);
// if you pass CFTreeNodeRef pParentNode NULL, you should get root item
EXTERN_API(CFTreeNodeRef)	GetTreeControlNodeAtIndex(TreeControlRef pTreeControlRef,CFTreeNodeRef pParentNode,int nIndex);
//Requests a redraw of one or more items in a tree control
EXTERN_API(void)			UpdateTreeControlNodes(TreeControlRef pTreeControlRef,CFTreeNodeRef pContainer,UInt32 numItems,const CFTreeNodeRef *items);
EXTERN_API(OSErr)			EmbedTreeControl(TreeControlRef pTreeControlRef,ControlRef pContainer);

EXTERN_API(UInt16)			GetTreeControlRowHeight(TreeControlRef pTreeControlRef,UInt16* height);
EXTERN_API(void)			DisposeTreeControl(TreeControlRef pTreeControlRef);
EXTERN_API(OSErr)			SetTreeControlColumnWidth(TreeControlRef pTreeControlRef,UInt16 width);
EXTERN_API(OSErr)			GetTreeControlColumnWidth(TreeControlRef pTreeControlRef,UInt16* width);

EXTERN_API(OSErr)			OpenTreeControlNode(TreeControlRef pTreeControlRef,CFTreeNodeRef pNode);
EXTERN_API(OSErr)			CloseTreeControlNode(TreeControlRef pTreeControlRef,CFTreeNodeRef pNode);

EXTERN_API(OSErr)			SelectTreeControlNode(TreeControlRef pTreeControlRef,CFTreeNodeRef pNode);
//treecontrol callback
EXTERN_API(void)			SetTreeControlCallBack(TreeControlRef pTreeControlRef,const TreeControlCallbacks* pTreeControlCallbacks);
EXTERN_API(void)			GetTreeControlCallBack(TreeControlRef pTreeControlRef,TreeControlCallbacks* pTreeControlCallbacks);
//treecontrol reference
EXTERN_API(void)			SetTreeControlReference(TreeControlRef pTreeControlRef,SInt32 data);
EXTERN_API(SInt32)			GetTreeControlReference(TreeControlRef pTreeControlRef);
// if you pass CFTreeNodeRef pParentNode NULL, TreeControl should insert pChildNode into root item
EXTERN_API(OSErr)			InsertTreeControlNode(TreeControlRef pTreeControlRef,CFTreeNodeRef pParentNode,CFTreeNodeRef pChildNode);
// if you pass CFTreeNodeRef pParentNode NULL, TreeControl should remove pChildNode into root item

EXTERN_API(OSErr)			RemoveTreeControlNode(TreeControlRef pTreeControlRef,CFTreeNodeRef pParentNode,CFTreeNodeRef pChildNode);

// if you pass CFTreeNodeRef pParentNode NULL, TreeControl should remove all form root item
EXTERN_API(OSErr)			RemoveAllTreeControlNode(TreeControlRef pTreeControlRef,CFTreeNodeRef pParentNode);

// base operation			
EXTERN_API(OSErr)			InstallTreeControlEventHandler(TreeControlRef		  pTreeControlRef,
														   EventHandlerUPP        inHandler,
														   UInt32                 inNumTypes,
														   const EventTypeSpec *  inList,
														   void *                 inUserData);
EXTERN_API(ControlRef)		GetTreeControl(TreeControlRef pTreeControlRef);
EXTERN_API(void)			ShowTreeControl(TreeControlRef pTreeControlRef,Boolean bShow);
EXTERN_API(void)			EnableTreeControl(TreeControlRef pTreeControlRef,Boolean bEnable);
EXTERN_API(void)			ActivateTreeControl(TreeControlRef pTreeControlRef,Boolean bActive);
EXTERN_API(Boolean)			IsTreeControlEnable(TreeControlRef pTreeControlRef);
EXTERN_API(Boolean)			IsTreeControlVisible(TreeControlRef pTreeControlRef);
EXTERN_API(Boolean)			IsTreeControlActive(TreeControlRef pTreeControlRef);
EXTERN_API(void)			MoveTreeControl(TreeControlRef pTreeControlRef,SInt16 hor, SInt16 ver);
EXTERN_API(void)			SizeTreeControl(TreeControlRef pTreeControlRef,SInt16 width, SInt16 height);
EXTERN_API(Boolean)			GetTreeControlBounds(TreeControlRef pTreeControlRef,Rect* pRect);
EXTERN_API(void)			SetTreeControlBounds(TreeControlRef pTreeControlRef,const Rect* pRect);
EXTERN_API(void)			SetTreeControlTitle(TreeControlRef pTreeControlRef,CFStringRef pTitle);
EXTERN_API(CFStringRef)		CopyTreeControlTitle(TreeControlRef pTreeControlRef);


enum
{
	kFilter_File = 1,
	kFilter_Package	= 1 << 1,
	kFilter_LocalDisk = 1 << 2,
	kFilter_RemoteDisk = 1 << 3,
	kFilter_DiscDisk = 1 << 4,
	kFilter_RemovableDisk = 1 << 5,
};

EXTERN_API(TreeControlRef)	CreateOSTreeControl(WindowRef pWindowRef,Rect* pRect,TreeControlStyle style, long filter);

// call it after call UpdateOSTreeControl
EXTERN_API(Boolean)			GetOSTreeControlSelectedPath(TreeControlRef pTreeControlRef,FSRef* pSelectRef);
EXTERN_API(Boolean)			SetOSTreeControlSelectedPath(TreeControlRef pTreeControlRef,FSRef* pSelectRef);

#endif //_H_TREECONTROL_









